/*
ESUP-portail is a french academic project developed under the GPL (General Public License) augmented according to the following :
A binary or source file developped by ESUP-portail can be used or compiled with products under Apache license.
The official english text of the GPL can be found here : http://www.gnu.org/licenses/gpl.html .
A non official french translation can be found here : http://www.linux-France.org/article/these/gpl.html .
The different kinds of licenses governing the products developed by the Apache foundation can be found here : http://www.apache.org/licenses .
It follows that you can as well as use download contents as well modify and redistribute them provided you respect the GPL terms.
Downloading and using such contents do not provide any guaranty.
Be sure that you have well understood the terms of the license before using the contents it covers.
The ESUP-portail distribution includes the following distributions :
    * UPortal : 
      software developed by JA-SIG (Java Architecture - Special Interest Group)
      You can find the license page here : http://mis105.udel.edu/ja-sig/uportal/license.html
    * CAS :
      SSO solution developed by Yale University
      You can find the project page here : http://www.yale.edu/tp/auth
    * Cocoon :
      XML framework distributed by the Apache foundation under Apache license;
      Please find the full text here : http://cocoon.apache.org/2.1/license.html
    * Mod_dav:
      A DAV module for Apache web server
      You can find the project page here : http://www.webdav.org/mod_dav
    * IMP :
      webmail from Horde application framework
      You can find the project page here : http://www.horde.org
    * . To be completed
*/
 
package org.esupportail.portal.channels.CWebdav;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.esupportail.portal.channels.CWebdav.config.Config;
import org.esupportail.portal.channels.CWebdav.config.Space;
import org.esupportail.portal.channels.CWebdav.exception.ApplicationException;
import org.esupportail.portal.channels.CWebdav.exception.BadFormatException;
import org.esupportail.portal.channels.CWebdav.exception.CancelException;
import org.esupportail.portal.channels.CWebdav.exception.CreateDirectoryException;
import org.esupportail.portal.channels.CWebdav.exception.DownloadException;
import org.esupportail.portal.channels.CWebdav.exception.NoneSelectedResourceException;
import org.esupportail.portal.channels.CWebdav.exception.NotAuthorizedException;
import org.esupportail.portal.channels.CWebdav.exception.NotAuthorizedNewDirException;
import org.esupportail.portal.channels.CWebdav.exception.NotEmptyDirectoryException;
import org.esupportail.portal.channels.CWebdav.exception.NotExistsResourceException;
import org.esupportail.portal.channels.CWebdav.exception.PasteDeletedResourceException;
import org.esupportail.portal.channels.CWebdav.exception.PropertiesException;
import org.esupportail.portal.channels.CWebdav.exception.ServerException;
import org.esupportail.portal.channels.CWebdav.exception.StillExistsException;
import org.esupportail.portal.channels.CWebdav.exception.TooMuchSelectedException;
import org.esupportail.portal.channels.CWebdav.exception.WebdavException;
import org.esupportail.portal.channels.CWebdav.provider.ChannelWebdavResource;
import org.esupportail.portal.channels.CWebdav.provider.DavAccess;
import org.esupportail.portal.channels.CWebdav.renderXml.RenderXmlShowCurrentDir;
import org.jasig.portal.ChannelCacheKey;
import org.jasig.portal.ChannelStaticData;
import org.jasig.portal.ICacheable;
import org.jasig.portal.IMimeResponse;
import org.jasig.portal.MultipartDataSource;
import org.jasig.portal.PortalControlStructures;
import org.jasig.portal.PortalException;
import org.jasig.portal.channels.BaseChannel;
import org.jasig.portal.security.IPerson;
import org.jasig.portal.services.LogService;
import org.jasig.portal.utils.XSLT;
import org.xml.sax.ContentHandler;

/**
 * Id: CWebdav.java,v 1.0 7 avr. 2004<br/>
 * Copyright (c) 2004 Esup Portail (www.esup-portail.org)<br/>
 * Classes: CWebdav<br/>
 * Original Author: Yohan Colmant<br/>
 * Channel used to manage personal files on a webdav server through the portal<br/>
 */




public class CWebdav extends BaseChannel implements IMimeResponse,ICacheable  {

  private ChannelStaticData staticData;
  
  private String fileId;
  private PortalControlStructures pcs;


  	/**
  	 * The config object
  	 */
  	private Config config;
  
  
	/**
	 * The actual mod in which is the channel
	 */
	private String currentMode;



	/**
	 * The stylesheet used
	 */
	private String stylesheet = "";
	
	
	
	/**
	 * The spaces availables
	 */
	private ArrayList spaces;
	
	/**
	 * The current used space
	 */
	private Space currentSpace;
	
	
	
	
	
	/**
	 * Error at the beginning of the channel
	 */
	private WebdavException startException;
	
	/**
	 * Information message
	 */
	private String informationMessage;
	
	/**
	 * Files/directories to delete
	 */
	private String[] filesToDelete;

	
	
	
	
	/**
	 * Files/directories to copy or move
	 */
	private Vector clipboard;
	
	/**
	 * The path where are the resource to copy or move
	 */
	private String clipboardPath;
	
	/**
	 * Param used to set if the selected target is to copy or to move
	 */
	private String clipboardParameter;
	
	
	
	
	/**
	 * Files/directories to rename
	 */
	private String fileToRename;

	
	/**
	 * User login
	 */
	private String login;
	


	/**
	 * The attribute used to access the webdav
	 */
	private DavAccess access;
	
	
	
	
	/**
	 * Constructor
	 */
	public CWebdav() {				
		
		// we begin in this mode
		this.currentMode = Constants.SHOW_CURRENT_DIR_MODE;
					
	}	

	
	/**
	 * Receive static channel data from the portal. Satisfies implementation of IChannel Interface.
	 * 
	 * @param sd <b>ChannelStaticData</b> static channel data
	 */
	public void setStaticData(ChannelStaticData sd) {		
		
		this.staticData = sd;	
		
		try {
			
			config = new Config();
			
			IPerson person = sd.getPerson();
			
			// if there is a trusted login
			String trustedLogin = config.getTrustedLogin();
			
			// the usernameAttribute
			String userNameAttribute = config.getUsernameAttribute();
						
			if (trustedLogin==null || trustedLogin.equals("")) {								
				
				
				//Set the user login			
				login = (String)person.getAttribute(userNameAttribute);			
			}
			else login = trustedLogin;
						
			// the password
			String password = config.getTrustedPassword();
			
			
			// the uri used for the connexion with webdav
			String uri = config.getUrl();		
			
			// Implementation of the access class
			String davAccessClass = config.getClassAccess();
			Class c = Class.forName(davAccessClass);
			access = (DavAccess)c.newInstance();							
			access.init(uri, login, password);
						
			
			// the spaces availables
			spaces = config.getSpaces();
			
			// for each Space, we check if the path is in the {...} form. Is yes, we replace {...} by the real value
			// after that, we try to obtain the redirected url. for example, if the path is /server/~bidon, we try to have the real path: for example /slide/files/storage/users/n/no/bidon
			access.connect();
			String serverCurrentHierPath = access.getServerCurrentHierPath();
			for (int i=0; i<spaces.size(); i++) {
				
				Space space = (Space)spaces.get(i);
				
				// check the {...}
				if (trustedLogin==null || trustedLogin.equals("")) {					
					space.checkPath(person, serverCurrentHierPath, null, null);
				}
				else {					
					space.checkPath(person, serverCurrentHierPath, userNameAttribute, trustedLogin);
				}
				
				// look for the redirection path
				String path = space.getPath();
				String newPath = access.getRedirectionPath(path);								
				
				// if path has changed
				if (!newPath.equals(path))
					space.setPath(newPath);										
			}						
			
						
			// at the launch, we check if each space can be showed or not			
			for (int i=0; i<spaces.size(); i++) {				
				Space space = (Space)spaces.get(i);				
				if (!space.canRead(access)) {
					spaces.remove(i);
					i--;					
				}								
			}			
			access.disconnect();
			
			// the current space
			this.currentSpace = (Space)spaces.get(0);									

		}
		catch(ServerException e) {			
			access = null;			
			startException = e;
			LogService.log(LogService.ERROR, "CWebdav::CWebdav::setStaticData : "+ e);
		}
		catch(ApplicationException e) {			
			access = null;			
			startException = e;
			LogService.log(LogService.ERROR, "CWebdav::CWebdav::setStaticData : "+ e);
		}
		catch(PropertiesException e) {			
			access = null;			
			startException = e;
			LogService.log(LogService.ERROR, "CWebdav::CWebdav::setStaticData : "+ e);
		}
		catch(ClassNotFoundException e) {			
			access = null;			
			startException = new PropertiesException();
			LogService.log(LogService.ERROR, "CWebdav::CWebdav::setStaticData : "+ e);
		}
		catch(IllegalAccessException e) {			
			access = null;			
			startException = new PropertiesException();
			LogService.log(LogService.ERROR, "CWebdav::CWebdav::setStaticData : "+ e);
		}
		catch(InstantiationException e) {			
			access = null;			
			startException = new PropertiesException();
			LogService.log(LogService.ERROR, "CWebdav::CWebdav::setStaticData : "+ e);
		}
		catch(MalformedURLException e) {			
			access = null;			
			startException = new PropertiesException();
			LogService.log(LogService.ERROR, "CWebdav::CWebdav::setStaticData : "+ e);
		}		
	}
	
	
	
	
		
	
	/**
	 * Output channel content to the portal
	 * 
	 * @param out a sax document handler
	 */
	public void renderXML(ContentHandler out) throws PortalException {	
				
		// set the mod
		// in the channel, there are 2 action menus. 
		// those menus have the same name modeDav. 
		// To differentiate the menu that we have selected, we check the value of the both. 
		// the menu with a not null value is the one we have selected.  
		currentMode = null;
		String[] mod=runtimeData.getParameterValues("modeDav");
		if (mod != null){
			for (int i = 0; i<mod.length;i++){
				if (!mod[i].equals("")){currentMode = mod[i];}
			}
		}
		
		
		// if we change the space
		String[] spaceChanges = runtimeData.getParameterValues("space");
		if (spaceChanges != null){
			String oldSpaceKey = this.currentSpace.getKey();
			String newSpaceKey = null;
			boolean findChangement = false;
			for (int i = 0; i<spaceChanges.length && !findChangement;i++){
				newSpaceKey = spaceChanges[i];
				if (!newSpaceKey.equals(oldSpaceKey))
					findChangement = true;
			}
			
			// if found, we look for the good space into the ArrayList
			if (findChangement) {
				
				boolean findGoodSpace = false;
				for (int i=0; i<spaces.size() && !findGoodSpace; i++) {
					Space s = (Space)spaces.get(i);
					if (s.getKey().equals(newSpaceKey)) {
						findGoodSpace = true;
						this.currentSpace = s;
					}
						
				}
				
			}
		}
		

		// Xml string of the channel
		StringBuffer xml = new StringBuffer(); 
		xml.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>");
		
		xml.append("<ROOT>");
		
		stylesheet = "";

		// Xml string generated by the diferents methods
		StringBuffer xmlTemp = new StringBuffer();
		
		try {
			
			// connect to the dav
			if (startException==null) {
				try {
					access.connect();
				}
				catch(Exception e) {
					startException= new ServerException();
				}
			}
			
			// set the default mod
			if (currentMode == null) {			
				currentMode = Constants.SHOW_CURRENT_DIR_MODE;
			}

			//////////////////////////////////////////////////////
			// D I R E C T O R Y
			//Display the current directory
			if (currentMode.equals(Constants.SHOW_CURRENT_DIR_MODE)) {	
				xmlTemp = renderXmlShowCurrentDir();			
				stylesheet = "CWebdav";
			}
			//Go to the parent directory
			else if (currentMode.equals(Constants.DIRECTORY_BACK)) {	
				directoryBack();
				xmlTemp = renderXmlShowCurrentDir();			
				stylesheet = "CWebdav";
			}
			//prepare create
			else if (currentMode.equals(Constants.SET_NAME_FOR_CREATE_DIR_MODE)) {		
				stylesheet = "newDirectory";						
			}
			//Real create
			else if (currentMode.equals(Constants.CREATE_DIR_MODE)) {		
				stylesheet = "CWebdav";
				createDir();		
				xmlTemp = RenderXmlShowCurrentDir.getXml(access, spaces, currentSpace, null, "2004", config);
			}

			//////////////////////////////////////////////////////
			// U P L O A D
			//prepare
			else if (currentMode.equals(Constants.SET_FILE_TO_UPLOAD_MODE)) {										
				stylesheet = "upload";
			} 
			//Real upload
			else if (currentMode.equals(Constants.UPLOAD_MODE)) {
				upload();
				stylesheet = "CWebdav";
				xmlTemp = RenderXmlShowCurrentDir.getXml(access, spaces, currentSpace, null, "2001", config);
			} 


			//////////////////////////////////////////////////////
			//D E L E T E
			//confirm
			else if (currentMode.equals(Constants.CONFIRM_DELETE_MODE)) {					
				StringBuffer x = renderXmlConfirmDelete();
				//
				if (config.getConfirmDel()) {
					stylesheet = "delete";
					xmlTemp = x;
				}
				
				// sinon, on passe directement au mode supprimer
				else {
					deleteWithoutConfirm();
					stylesheet = "CWebdav";
					xmlTemp = RenderXmlShowCurrentDir.getXml(access, spaces, currentSpace, null, "2002", config);
				}													
			}
			//Real delete
			else if (currentMode.equals(Constants.DELETE_MODE)) {										
				delete();
				stylesheet = "CWebdav";			
				xmlTemp = RenderXmlShowCurrentDir.getXml(access, spaces, currentSpace, null, "2002", config);
			}

			//////////////////////////////////////////////////////  
			//C O P Y
			//set clipboard
			else if (currentMode.equals(Constants.SET_CLIPBOARD_FOR_COPY_MODE)) {			
				stylesheet = "CWebdav";				
				setClipboard(Constants.COPY_MODE);
				xmlTemp = RenderXmlShowCurrentDir.getXml(access, spaces, currentSpace, null, "2007", config);
			}
			
			// ask if we paste really
			else if (currentMode.equals(Constants.CONFIRM_PASTE_MODE)) {			
				stylesheet = "copier";								
				xmlTemp = renderXmlConfirmPaste();
			}
			
			//Real paste
			else if (currentMode.equals(Constants.PASTE_MODE)) {
				stylesheet = "CWebdav";
				xmlTemp = paste();						
			}

			//////////////////////////////////////////////////////
			//M O V E
			//set clipboard
			else if (currentMode.equals(Constants.SET_CLIPBOARD_FOR_MOVE_MODE)) {			
				stylesheet = "CWebdav";
				setClipboard(Constants.MOVE_MODE);
				xmlTemp = RenderXmlShowCurrentDir.getXml(access, spaces, currentSpace, null, "2007", config);
			}			

			//////////////////////////////////////////////////////
			//R E N A M E
			//prepare
			else if (currentMode.equals(Constants.SET_NAME_FOR_RENAME_MODE)) {
				stylesheet = "rename";
				xmlTemp = renderXmlSetNameForRename();		
			}
			//Real rename
			else if (currentMode.equals(Constants.RENAME_MODE)) {				
				rename();
				stylesheet = "CWebdav";	
				xmlTemp = RenderXmlShowCurrentDir.getXml(access, spaces, currentSpace, null, "2003", config);
			} 
			
			xml.append(xmlTemp);			
				
		}		
		catch(WebdavException e) {
			stylesheet = "CWebdav";
			xml.append(RenderXmlShowCurrentDir.getXml(access, spaces, currentSpace, e, null, config));
		}
		
		try {
			// discconnect from the dav
			access.disconnect();
		}
		catch (Exception e) {
			LogService.log(LogService.ERROR, "CWebdav::renderXml "+e);
		}
		
		
		xml.append("</ROOT>");

		//Create a new XSLT styling engine
		XSLT xslt = new XSLT(this);

		//pass the result XML to the styling engine.
		xslt.setXML(xml.toString());

		//specify the stylesheet selector		
		xslt.setXSL("CWebdav.ssl", stylesheet, runtimeData.getBrowserInfo());
		
		
		//url to the channel
		xslt.setStylesheetParameter("baseActionURL",runtimeData.getBaseActionURL());
		//url to the download worker
		xslt.setStylesheetParameter( "downloadURL",runtimeData.getBaseWorkerURL( org.jasig.portal.UPFileSpec.FILE_DOWNLOAD_WORKER,true).replaceAll("%2F","/"));
		
		// the parameter to know if there is a resource in the clipboard
		if (this.clipboard != null)
			xslt.setStylesheetParameter("isClipboardEmpty","false");			      
		
		// the parameter used in the copy/move mode. tell if the mode is copy or move
		if (clipboardParameter != null) {
			xslt.setStylesheetParameter("clipboardParameter",clipboardParameter);			

		}
		
		// set the output Handler for the output.
		xslt.setTarget(out);
	
		// do the deed
		xslt.transform();			
	}
	
	
	
	
	/**
	 * Generate the XML String for this mod
	 * @return le XML pour ce mode
	 */
	private StringBuffer renderXmlShowCurrentDir() {	
				
		startException = null;
		informationMessage = null;
		
		//When we enter in a sub directory ...
		String directoryCible = runtimeData.getParameter("directory");
		if (directoryCible != null) {			
			try {				
				directoryCible = URLDecoder.decode(directoryCible,"ISO-8859-1");
			} catch (UnsupportedEncodingException e1) {				
				e1.printStackTrace();
			}
			return RenderXmlShowCurrentDir.getXml(access, spaces, currentSpace, directoryCible, startException , informationMessage, config);
		}
		else {
			//... by naviguation tool
			String dir = runtimeData.getParameter("link");
			if (dir != null) {
				int index = Integer.parseInt(dir);
				int curentDirLength = currentSpace.getPathSize();
				if (index < curentDirLength) {					
					for (int i=0; i< (curentDirLength-(index+1)); i++) {						
						currentSpace.removePathElement(currentSpace.getPathSize()-1);
					}			
				}
				return RenderXmlShowCurrentDir.getXml(access, spaces, currentSpace, startException , informationMessage, config);
			}
		}
					
		// all others cases
		return RenderXmlShowCurrentDir.getXml(access, spaces, currentSpace, startException , informationMessage, config);
	}
	
	
	/**
	 * Go to the parent directory
	 */
	private void directoryBack() {
		if (currentSpace.getPathSize()>1)
			currentSpace.removePathElement(currentSpace.getPathSize()-1);		
	}
	
	
	
	/**
	 * Start the upload
	 */
	private void upload() throws WebdavException {
	       
		String submit = runtimeData.getParameter("Submit");
		String cancel = runtimeData.getParameter("Cancel");
		Object inputFile = this.runtimeData.getObjectParameter("input_file");        
	
		if (submit == null) {		
			throw new CancelException();		       		       	
		}
		else {			
			// if file is null
			if (inputFile == null) {				
				throw new NotExistsResourceException();
			}			
			
			String information = null;
			String error = null;
			
			// get a reference to the file
			MultipartDataSource mps = (MultipartDataSource)inputFile;   	        	
			
	        String path = currentSpace.getPath();			
			access.upload(mps, path);
		}
	}

	
	
	/**
	 * Generate the XML String for this mod
	 * @return le XML pour ce mode
	 */
	private StringBuffer renderXmlConfirmDelete() throws WebdavException {
		// Xml generated
		StringBuffer xml = new StringBuffer();
				
		filesToDelete = runtimeData.getParameterValues("listeFic");
		if (filesToDelete == null) {
			throw new NoneSelectedResourceException();			
		}
		
		else {
			//check if it is allowed to delete non empty directory
			boolean authorise = config.getAllowDelNonEmptyFolder();		

			//For each piece of the selection
			for (int j=0;j<filesToDelete.length;j++){
				String fileName = filesToDelete[j];				

				String path = currentSpace.getPath();			
				
				//check if it is a directory or a file
				boolean isDirectory = access.isDirectory(fileName, path);																
				if (isDirectory) {	
					
					//check if non empty
					boolean empty = access.isEmpty(fileName, path);														
										
					if (!empty && !authorise) {
						//Delete forbidden for this directory
						filesToDelete = null;
						throw new NotEmptyDirectoryException();
					}
					else if (!empty && authorise)
						xml.append("<DIR name=\""+fileName+"\" empty=\""+false+"\"/>");
					else if (empty)											
						xml.append("<DIR name=\""+fileName+"\" empty=\""+true+"\"/>");
				}
				
				else if (!isDirectory)					
					xml.append("<FILE name=\""+fileName+"\"/>");								
			}			
			return xml;					
		}
	}
	
		
	
	
	
	/**
	 * Generate the XML String for this mod
	 * @return the XML String for this mod
	 */
	private void delete() throws WebdavException  {
		String submit = runtimeData.getParameter("Submit");
		String cancel = runtimeData.getParameter("Cancel");
		
		if (submit == null) {
			throw new CancelException();
		}
		
		String information = null;
		String error = null;
			
		String path = currentSpace.getPath();	
		
		for (int i=0; i<filesToDelete.length; i++) {
			String file = (String)filesToDelete[i];
			access.delete(file, path);				
		}
			
		filesToDelete = null;								
	} 
	
	
	/**
	 * Generate the XML String for this mod
	 * @return the XML String for this mod
	 */
	private void deleteWithoutConfirm() throws WebdavException {		
		
		String information = null;
		String error = null;
			
		String path = currentSpace.getPath();		
		
		for (int i=0; i<filesToDelete.length; i++) {
			String file = (String)filesToDelete[i];
			access.delete(file, path);				
		}
			
		filesToDelete = null;								
	}
	
	
	
	/**
	 * Generate the XML String for this mod
	 * @return the XML String for this mod
	 */
	private void setClipboard(String mode) throws NoneSelectedResourceException, ApplicationException, NotAuthorizedException {			
		
		//check the files/directories from parameters
		String[] files = runtimeData.getParameterValues("listeFic");
		if (files == null) {			
			throw new NoneSelectedResourceException();
		}		
		else {
			
			// we check if we are authorized to copy all selected resources			
			String tempPath = currentSpace.getPath();
			for (int j=0;j<files.length;j++){
				String fileName = files[j];
				if (!access.canRead(fileName, tempPath)) {
					// we erase the clipboard
					clipboard = null;
					clipboardParameter = null;
					clipboardPath = null;
					
					// we throw an exception
					throw new NotAuthorizedException();
				}
			}
			
			// if none NotAuthorizedException throwed
			
			// files/directories to copy
			clipboard = new Vector();
			clipboardPath = currentSpace.getPath();
			
			for (int j=0;j<files.length;j++){
				String fileName = files[j];
				clipboard.addElement(fileName);				
			}			
		}
		
		clipboardParameter = mode;		
	}
	
	
	/**
	 * Generate the XML String for this mod
	 * @return the XML String for this mod
	 */
	private StringBuffer renderXmlConfirmPaste() throws WebdavException {
		
		// we check if we can paste here
		try {
			access.canPaste(this.clipboardPath, this.clipboard, currentSpace.getPath());
		}
		catch (PasteDeletedResourceException e) {
			clipboard = null;
			clipboardParameter = null;
			clipboardPath = null;
			throw new PasteDeletedResourceException();
		}
		
		// Xml generated
		StringBuffer xml = new StringBuffer();
		
		//For each piece of the selection
		for (int j=0;j<clipboard.size();j++){
			String fileName = (String)clipboard.elementAt(j);				
			
			//check if it is a directory or a file
			boolean isDirectory = access.isDirectory(fileName, clipboardPath);								
						
			if (isDirectory) {	
					
				//check if non empty
				boolean empty = access.isEmpty(fileName, clipboardPath);														
				
				if (!empty)
					xml.append("<DIR name=\""+fileName+"\" empty=\""+false+"\"/>");
				else if (empty)											
					xml.append("<DIR name=\""+fileName+"\" empty=\""+true+"\"/>");
			}				
			else if (!isDirectory)					
				xml.append("<FILE name=\""+fileName+"\"/>");								
		}
			
		return xml;							
	}
	
	
	/**
	 * Generate the XML String for this mod
	 * @return the XML String for this mod
	 */
	private StringBuffer paste() throws WebdavException {
		String submit = runtimeData.getParameter("Submit");
		String cancel = runtimeData.getParameter("Cancel");
		
		if (cancel == null) {			
			for (int i=0; i<clipboard.size(); i++) {								
				if (clipboardParameter.equals(Constants.COPY_MODE)) {			
					access.copy(((String)clipboard.elementAt(i)), clipboardPath, currentSpace.getPath());
				}
				else {					
					access.move(((String)clipboard.elementAt(i)), clipboardPath, currentSpace.getPath());
				}						
			}			
		} 		
		else {
			throw new CancelException();
		}				
		
		// if well done
		if (clipboardParameter.equals(Constants.COPY_MODE)) {			
			/*
			// if we want to set empty the clipboard
			clipboard = null;
			clipboardParameter = null;
			clipboardPath = null;
			*/
			return RenderXmlShowCurrentDir.getXml(access, spaces, currentSpace, null, "2005", config);	
		}
		else {
			clipboard = null;
			clipboardParameter = null;
			clipboardPath = null;
			return RenderXmlShowCurrentDir.getXml(access, spaces, currentSpace, null, "2006", config);	
		}
	}
	
	
	/**
	 * Generate the XML String for this mod
	 * @return the XML String for this mod
	 */
	private void move() throws WebdavException {
		String submit = runtimeData.getParameter("Submit");
		String cancel = runtimeData.getParameter("Cancel");
		
		String target = "/"+runtimeData.getParameter("target");
				
		if (cancel == null) {
			String fromPath = currentSpace.getPath();
				
			for (int i=0; i<clipboard.size(); i++) {								
				access.move(((String)clipboard.elementAt(i)), fromPath, target);			
			}
		} 
		
		else {
			throw new CancelException();
		}				
	}
	
	
	/**
	 * Generate the XML String for this mod
	 * @return the XML String for this mod
	 */
	private StringBuffer renderXmlSetNameForRename() {
		// we get the files we want to rename
		String[] files = runtimeData.getParameterValues("listeFic");
		
		if (files == null) {
			stylesheet = "CWebdav";
			return RenderXmlShowCurrentDir.getXml(access, spaces, currentSpace, new NoneSelectedResourceException(), null, config);
		}		
		else {						
			// if there is more than one file
			if (files.length > 1) {
				stylesheet = "CWebdav";
				return RenderXmlShowCurrentDir.getXml(access, spaces, currentSpace, new TooMuchSelectedException(), null, config);
			}
			else {
				fileToRename = files[0];
				StringBuffer sb = new StringBuffer();
				sb.append("<OLD>");
				sb.append(fileToRename);
				sb.append("</OLD>");
				return sb;
			}					
		}
	}
	
	
	/**
	 * Generate the XML String for this mod
	 * @return the XML String for this mod
	 */
	private void rename() throws WebdavException{
		String submit = runtimeData.getParameter("Submit");
		String cancel = runtimeData.getParameter("Cancel");
		
		if (submit == null) {
			fileToRename = null;
			throw new CancelException();
		}
		else {
			String information = null;
			String error = null;
				
			// we clicked on the validate button: we rename 
			String path = currentSpace.getPath();		
						
			String newName = runtimeData.getParameter("new_name");
			access.rename(fileToRename, newName, path);
			
			fileToRename = null;						
		}
	}
			
	
	
	/**
	 * Generate the XML String for this mod
	 * @return the XML String for this mod
	 */
	private void createDir() throws CancelException, ApplicationException, CreateDirectoryException, StillExistsException, BadFormatException, NotAuthorizedException, NotAuthorizedNewDirException, NotExistsResourceException{
		String submit = runtimeData.getParameter("Submit");
		String cancel = runtimeData.getParameter("Cancel");
		
		if (submit == null) {
			throw new CancelException();
		} else {
			String information = null;
			String error = null;
				
			String path = currentSpace.getPath();
			int errorCode = 0;

			String newDirName = runtimeData.getParameter("new_dir");
			access.createDir(newDirName, path);
        	
			LogService.log(LogService.DEBUG, "CWebdav::createDir: newDir = "+newDirName+"******* path = "+path);
			
		}
	}




/***************************************************************************/
/*********************START OF DOWNLOAD FUNCTIONS***************************/
/***************************************************************************/

	/**
	 * The resource to download
	 */
	private ChannelWebdavResource downloadResource = null;
	
	
  	/**
	 * Return the content type of the seleted file
	 * @see org.jasig.portal.IMimeResponse#getContentType()
	 */
	public String getContentType() {
		return downloadResource.getContentType();
	}


	/**
	 * Send HTTP header
	 * @see org.jasig.portal.IMimeResponse#getHeaders()
	 */
	public Map getHeaders() {
		Map map = new HashMap();
      
		fileId = runtimeData.getParameter("file");
		try {
			fileId = URLDecoder.decode(fileId,"UTF-8");
		}
		catch (UnsupportedEncodingException e) {
			LogService.log(LogService.ERROR,"CWebdav::getHeaders: UnsupportedEncodingException"+e);
		}
    
		// we check if it is a directory
		String path = currentSpace.getPath();
		
		try {
			downloadResource = access.dowload(fileId,path);
				
			fileId = fileId.replaceAll(" ", "_");
			fileId = fileId.replaceAll("[]", "a");
			fileId = fileId.replaceAll("[]", "e");
			fileId = fileId.replaceAll("[]", "i");
			fileId = fileId.replaceAll("[]", "o");
			fileId = fileId.replaceAll("[]", "u");
			fileId = fileId.replaceAll("", "c");
			fileId = fileId.replaceAll("", "n");
			fileId = fileId.replaceAll("[]", "A");
			fileId = fileId.replaceAll("[]", "E");
			fileId = fileId.replaceAll("[]", "I");
			fileId = fileId.replaceAll("[]", "O");
			fileId = fileId.replaceAll("[]", "U");
			fileId = fileId.replaceAll("", "C");
			fileId = fileId.replaceAll("", "N");		
			
			// open in new window : inline  -- ||
			// download : attachment  -- -- -- ||
			//                                 \/
			map.put("Content-disposition", "attachment; filename="+fileId  );

			return map;
		}
		catch(DownloadException e) {
			map.put("Location", runtimeData.getBaseActionURL());
			return map;
		}
	}
	
	
	/**
	 *	Get the strem for the file to dowload
	 * @see org.jasig.portal.IMimeResponse#getInputStream()
	 */
	public InputStream getInputStream() throws IOException {	
		return downloadResource.getInputStream();
	}


	/**
	 * Function unused here
	 * @see org.jasig.portal.IMimeResponse#downloadData(java.io.OutputStream)
	 */
	public void downloadData(OutputStream arg0) throws IOException {
	}


	/**
	 * Function unused here
	 * @see org.jasig.portal.IMimeResponse#getName()
	 */
	public String getName() {
		return null;
	}


	/**
	 * Print the error generated by the download
	 * @see org.jasig.portal.IMimeResponse#reportDownloadError(java.lang.Exception)
	 */
	public void reportDownloadError(Exception arg0) {
		arg0.printStackTrace();
	}

	

	/***************************************************************************/
	/*********************END OF DOWNLOAD FUNCTIONS*****************************/
	/***************************************************************************/



	/***************************************************************************/
	/**********************START OF CACHE FUNCTIONS*****************************/
	/***************************************************************************/

	/* (non-Javadoc)
	 * @see org.jasig.portal.ICacheable#generateKey()
	 */
	public ChannelCacheKey generateKey() {

	    ChannelCacheKey k = new ChannelCacheKey();
	    StringBuffer sbKey = new StringBuffer(1024);
	    sbKey.append(this.getClass().getName() + " : ");
	    sbKey.append("login : "+login+";");
	    sbKey.append("SubscribeId:"+this.staticData.getChannelSubscribeId());
	    k.setKeyScope(ChannelCacheKey.INSTANCE_KEY_SCOPE);

	    k.setKey(sbKey.toString());
	    k.setKeyValidity(new Long(System.currentTimeMillis()));
	    /*LogService.log(LogService.DEBUG,
	                              "CWebdav::generateKey() (id : " +
	                              this.staticData.getChannelSubscribeId() +
	                              ") : generateKey :  -> Cle de cache:" + k.getKey());
	    */
		return k;
	}

	/* (non-Javadoc)
	 * @see org.jasig.portal.ICacheable#isCacheValid(java.lang.Object)
	 */
	public boolean isCacheValid(Object validity) {

	    boolean cacheValid = false;

	  if(runtimeData.getParameters().size()==0)
	  		cacheValid=true;

	  /*if (cacheValid) {
        LogService.log(LogService.DEBUG,
                                  "CWebdav::isCacheValid (" +
                                  this.staticData.getChannelSubscribeId() +
                                  ") : isCacheValid -> YES");
      }
      else {
        LogService.log(LogService.DEBUG,
                                  "CWebdav::isCacheValid (" +
                                  this.staticData.getChannelSubscribeId() +
                                  ") : isCacheValid -> NO");
      }*/
	    return cacheValid;
	}
	/***************************************************************************/
	/************************END OF CACHE FUNCTIONS*****************************/
	/***************************************************************************/
}
